import javax.swing.*;

/**
	Dies ist die Klasse, in der die Architektur des Toy-Rechners
	veranschaulicht wird. Dabei wird ein vorher eingescanntes Bild der
	Architektur verwendet, hier werden darauf Textfelder abgebildet, die mit
	aktuellen Werten gefllt werden. 
*/
public class TArchGUI extends JFrame {
	private JPanel contentsPane = null;
	private ToyEmulatorKernel callingKernel = null;
	private boolean doNextStep=true;

	
	/**
		Im Konstruktor wird der aufrufende Kernel zur spteren
		Kommunikation gespeichert und einige Initialisierungsmethoden
		aufgerufen.
		@param tempcallingKernel: der aufrufende
		<tt>ToyEmulatorKernel</tt>
	*/
	public TArchGUI(ToyEmulatorKernel tempcallingKernel) {
		super();
		callingKernel=tempcallingKernel;
		setArchTextFields();
		doNextStep=false;
		initialize();
	}

	
	/**
		In dieser Initialisierungsmethode werden die Attribute des
		Architekturfensters auf sinnvolle Startwerte gesetzt.
	*/
	private void initialize() {
		try {
			setName("TArchGUI");
			setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
			setTitle("ToyEmulator Architecture");
			setSize(600, 430);
			setVisible(true);
			setResizable(false);
			setContentPane(getJFrameContentPane());
			initConnections();
		} catch (java.lang.Throwable ivjExc) {
			handleException(ivjExc);
		}
	}


	/**
		Diese Methode dient der Manipulation des Klassenattributes
		doNextStep von auen.
		@param value: neuer Wert von doNextStep
	*/
	public void setDoNextStep(boolean value)
	{
		doNextStep=value;
	}


	/**
		In dieser Methode werden die Textfelder des Architekturfensters
		aktualisiert. Dabei wird auf Werte des
		<tt>ToyEmulatorKernel</tt> zurckgegriffen.
	*/
	public void setArchTextFields()
	{
		String temp,temp2;
	
		// Phase 1
		if(doNextStep) {
			temp=callingKernel.getPCValue().substring(1);
			getpcMpTextField().setText(temp);
			getmpRamTextField().setText(temp);
			temp2=callingKernel.getArgumentFromMemory(Integer.parseInt(temp,16)).toString().substring(1);
			temp=callingKernel.getInsnregFromMemory().toString().substring(3);
			getramIrTextField().setText(temp+temp2);
			getramAluTextField().setText(temp+temp2);
			getirSwTextField().setText(temp);
			getirMpTextField().setText(temp2);
			getaccuAluTextField().setText(callingKernel.getAccuValue());
		//Phase 2
		} else {
			temp=callingKernel.getPCValue().substring(1);
			temp2=callingKernel.getArgumentFromMemory(Integer.parseInt(temp,16)).toString().substring(1);
			getmpRamTextField().setText(temp2);
			temp=callingKernel.getArgumentFromMemory(Integer.parseInt(temp2,16)).toString();
			getramAluTextField().setText(temp);
			getramIrTextField().setText(temp);
			temp2=callingKernel.preCalcAccuValue();
			getaluRamTextField().setText(temp2);
			getaluAccuTextField().setText(temp2);
			getaccuAluTextField().setText(callingKernel.getAccuValue());
		}	
	}	
	

	/**
		Hier wird der Hauptcontainer erzeugt und bei Bedarf
		zurckgegeben.
		@return Hauptcontainer des Fensters
	*/
	private JPanel getContentsPane() {
		if (contentsPane == null) {
			contentsPane = new javax.swing.JPanel();
			contentsPane.setName("ContentsPane");
			contentsPane.setLayout(null);
		
		}
		return contentsPane;
	}	

	


	/**
 		Wird aufgerufen, wenn die Komponente eine Ausnahmebedingung bergibt.
 		@param exception java.lang.Throwable
	*/                    
	private void handleException(java.lang.Throwable exception) {
		System.out.println("--------- NICHT ABGEFANGENE AUSNAHMEBEDINGUNG ---------");
		exception.printStackTrace(System.out);
	}
	
																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																																															private JTextField ivjaccuAluTextField = null;						private JTextField ivjaluAccuTextField = null;							private ImagePanel ivjImagePanel1 = null;						private JTextField ivjirMpTextField = null;						private JTextField ivjirSwTextField = null;						private JPanel ivjJFrameContentPane = null;									private JTextField ivjpcMpTextField = null;						private JTextField ivjramAluTextField = null;				

	/**
		Diese Methode generiert und liefert das Attribut
		accuAluTextField zurck. Es werden auerdem Initialisierungen
		vorgenommen.
		@return accuAluTextField
	*/
	public javax.swing.JTextField getaccuAluTextField() {
		if (ivjaccuAluTextField == null) {
			try {
				ivjaccuAluTextField = new javax.swing.JTextField();
				ivjaccuAluTextField.setName("accuAluTextField");
				ivjaccuAluTextField.setText("0000");
				ivjaccuAluTextField.setBounds(463, 111, 40, 25);
				ivjaccuAluTextField.setEditable(false);
				ivjaccuAluTextField.setColumns(4);
			} catch (java.lang.Throwable ivjExc) {
				handleException(ivjExc);
			}
		}
		return ivjaccuAluTextField;
	}


	/**
		Diese Methode generiert und liefert das Attribut
		aluAccuTextField zurck. Es werden auerdem Initialisierungen
		vorgenommen.
		@return aluAccuTextField
	*/
	private javax.swing.JTextField getaluAccuTextField() {
		if (ivjaluAccuTextField == null) {
			try {
				ivjaluAccuTextField = new javax.swing.JTextField();
				ivjaluAccuTextField.setName("aluAccuTextField");
				ivjaluAccuTextField.setText("0000");
				ivjaluAccuTextField.setBounds(462, 24, 40, 23);
				ivjaluAccuTextField.setEditable(false);
			} catch (java.lang.Throwable ivjExc) {
				handleException(ivjExc);
			}
		}	
		return ivjaluAccuTextField;
	}


	/**
		Diese Methode generiert und liefert das Attribut
		ImagePanel1 zurck. Es werden auerdem Initialisierungen
		vorgenommen sowie alle bentigten Komponenten hinzugefgt.
		@return ImagePanel1
	*/
	private ImagePanel getImagePanel1() {
		if (ivjImagePanel1 == null) {
			try {
				ivjImagePanel1 = new ImagePanel();
				ivjImagePanel1.setName("ImagePanel1");
				ivjImagePanel1.setLayout(null);
				ivjImagePanel1.setBounds(0, 0, 600, 430);
				getImagePanel1().add(getramIrTextField(), getramIrTextField().getName());
				getImagePanel1().add(getirSwTextField(), getirSwTextField().getName());
				getImagePanel1().add(getirMpTextField(), getirMpTextField().getName());
				getImagePanel1().add(getpcMpTextField(), getpcMpTextField().getName());
				getImagePanel1().add(getmpRamTextField(), getmpRamTextField().getName());
				getImagePanel1().add(getramAluTextField(), getramAluTextField().getName());
				getImagePanel1().add(getaccuAluTextField(), getaccuAluTextField().getName());
				getImagePanel1().add(getaluAccuTextField(), getaluAccuTextField().getName());
				getImagePanel1().add(getaluRamTextField(), getaluRamTextField().getName());
				getImagePanel1().add(getnextPhaseButton(), getnextPhaseButton().getName());
			} catch (java.lang.Throwable ivjExc) {
				handleException(ivjExc);
			}
		}
		return ivjImagePanel1;
	}




	/**
		Diese Methode generiert und liefert das Attribut
		irMpTextField zurck. Es werden auerdem Initialisierungen
		vorgenommen.
		@return irMpTextField
	*/
	private javax.swing.JTextField getirMpTextField() {
		if (ivjirMpTextField == null) {
			try {
				ivjirMpTextField = new javax.swing.JTextField();
				ivjirMpTextField.setName("irMpTextField");
				ivjirMpTextField.setText("000");
				ivjirMpTextField.setBounds(209, 152, 28, 26);
				ivjirMpTextField.setEditable(false);
				ivjirMpTextField.setColumns(3);
			} catch (java.lang.Throwable ivjExc) {
				handleException(ivjExc);
			}
		}
		return ivjirMpTextField;
	}


	/**
		Diese Methode generiert und liefert das Attribut
		irSwTextField zurck. Es werden auerdem Initialisierungen
		vorgenommen.
		@return irSwTextField
	*/
	private javax.swing.JTextField getirSwTextField() {
		if (ivjirSwTextField == null) {
			try {
				ivjirSwTextField = new javax.swing.JTextField();
				ivjirSwTextField.setName("irSwTextField");
				ivjirSwTextField.setText("0");
				ivjirSwTextField.setBounds(99, 154, 16, 25);
				ivjirSwTextField.setEditable(false);
				ivjirSwTextField.setColumns(1);
			} catch (java.lang.Throwable ivjExc) {
				handleException(ivjExc);
			}
		}
		return ivjirSwTextField;
	}


	/**
		Diese Methode generiert und liefert das Attribut
		JFrameContentPane zurck. Es werden auerdem Initialisierungen
		vorgenommen.
		@return JFrameContentPane
	*/
	private javax.swing.JPanel getJFrameContentPane() {
		if (ivjJFrameContentPane == null) {
			try {
				ivjJFrameContentPane = new javax.swing.JPanel();
				ivjJFrameContentPane.setName("JFrameContentPane");
				ivjJFrameContentPane.setLayout(null);
				ivjJFrameContentPane.setMaximumSize(new java.awt.Dimension(600, 430));
				ivjJFrameContentPane.setVisible(true);
				ivjJFrameContentPane.setPreferredSize(new java.awt.Dimension(600, 430));
				ivjJFrameContentPane.setMinimumSize(new java.awt.Dimension(600, 430));
				getJFrameContentPane().add(getImagePanel1(), getImagePanel1().getName());
			} catch (java.lang.Throwable ivjExc) {
				handleException(ivjExc);
			}
		}
		return ivjJFrameContentPane;
	}	





	/**
		Diese Methode generiert und liefert das Attribut
		pcMpTextField zurck. Es werden auerdem Initialisierungen
		vorgenommen.
		@return pcMpField
	*/
	private javax.swing.JTextField getpcMpTextField() {
		if (ivjpcMpTextField == null) {
			try {
				ivjpcMpTextField = new javax.swing.JTextField();
				ivjpcMpTextField.setName("pcMpTextField");
				ivjpcMpTextField.setText("000");
				ivjpcMpTextField.setBounds(312, 245, 29, 25);
				ivjpcMpTextField.setEditable(false);
			} catch (java.lang.Throwable ivjExc) {
				handleException(ivjExc);
			}
		}
		return ivjpcMpTextField;
	}


	/**
		Diese Methode generiert und liefert das Attribut
		ramAluTextField zurck. Es werden auerdem Initialisierungen
		vorgenommen.
		@return ramAluTextField
	*/
	private javax.swing.JTextField getramAluTextField() {
		if (ivjramAluTextField == null) {
			try {
				ivjramAluTextField = new javax.swing.JTextField();
				ivjramAluTextField.setName("ramAluTextField");
				ivjramAluTextField.setText("0000");
				ivjramAluTextField.setBounds(378, 69, 40, 24);
				ivjramAluTextField.setEnabled(true);
				ivjramAluTextField.setEditable(false);
			} catch (java.lang.Throwable ivjExc) {
				handleException(ivjExc);
			}
		}
		return ivjramAluTextField;
	}



	private JTextField ivjaluRamTextField = null;					
	IvjEventHandler ivjEventHandler = new IvjEventHandler();				
	
	
	class IvjEventHandler implements java.awt.event.ActionListener, java.awt.event.WindowListener {
		public void actionPerformed(java.awt.event.ActionEvent e) {
			if (e.getSource() == TArchGUI.this.getnextPhaseButton()) 
				connEtoC3(e);
		};
		public void windowActivated(java.awt.event.WindowEvent e) {};
		public void windowClosed(java.awt.event.WindowEvent e) {};
		public void windowClosing(java.awt.event.WindowEvent e) {
			if (e.getSource() == TArchGUI.this) 
				connEtoC1(e);
		};
		public void windowDeactivated(java.awt.event.WindowEvent e) {};
		public void windowDeiconified(java.awt.event.WindowEvent e) {};
		public void windowIconified(java.awt.event.WindowEvent e) {};
		public void windowOpened(java.awt.event.WindowEvent e) {};
	}
	
	
   

	/**
		Diese Methode wird aufgerufen, falls das Fenster geschlossen
		werden soll. Es ruft dann ber eine weitere Methode den
		aufrufenden Kernel auf, der das Fenster korrekt schliet.
		@param arg1: ausgelstes WindowEvent
	*/
	private void connEtoC1(java.awt.event.WindowEvent arg1) {
		try {
			this.tArchGUI_WindowClosing(arg1);
		} catch (java.lang.Throwable ivjExc) {
			handleException(ivjExc);
		}
	}
   


	/**
		Diese Methode generiert und liefert das Attribut
		aluRamTextField zurck. Es werden auerdem Initialisierungen
		vorgenommen.
		@return aluRamTextField
	*/
	private javax.swing.JTextField getaluRamTextField() {
		if (ivjaluRamTextField == null) {
			try {
				ivjaluRamTextField = new javax.swing.JTextField();
				ivjaluRamTextField.setName("aluRamTextField");
				ivjaluRamTextField.setText("0000");
				ivjaluRamTextField.setBounds(418, 200, 40, 24);
				ivjaluRamTextField.setEditable(false);
			} catch (java.lang.Throwable ivjExc) {
				handleException(ivjExc);
			}
		}
		return ivjaluRamTextField;
	}  

   
	/**
		Hier werden die Verknpfungen zwischen dem einzigen Knopf des
		Fensters und dem Fenster selbst hergestellt.
	*/
	private void initConnections() throws java.lang.Exception {
		this.addWindowListener(ivjEventHandler);
		getnextPhaseButton().addActionListener(ivjEventHandler);
	}

	/**
 		Diese Methode wird aufgerufen, wenn das Fenster geschlossen
		werden soll. Sie ruft anschlieend eine Methode von
		<tt>ToyEmulatorKernel</tt> auf, die sich um das Korrekte
		Schlieen des Fensters kmmmert.
		@param windowEvent: das auslsende Ereignis
 	*/         
	public void tArchGUI_WindowClosing(java.awt.event.WindowEvent windowEvent) {
		callingKernel.arch();
	};
	
	
	private JTextField ivjmpRamTextField = null;


	/**
		Diese Methode generiert und liefert das Attribut
		mpRamTextField zurck. Es werden auerdem Initialisierungen
		vorgenommen.
		@return mpRamTextField
	*/
	private javax.swing.JTextField getmpRamTextField() {
		if (ivjmpRamTextField == null) {
			try {
				ivjmpRamTextField = new javax.swing.JTextField();
				ivjmpRamTextField.setName("mpRamTextField");
				ivjmpRamTextField.setText("000");
				ivjmpRamTextField.setBounds(287, 344, 28, 27);
				ivjmpRamTextField.setEditable(false);
			} catch (java.lang.Throwable ivjExc) {
				handleException(ivjExc);
			}
		}
		return ivjmpRamTextField;
	}
	
	private JTextField ivjramIrTextField = null;

	/**
		Diese Methode generiert und liefert das Attribut
		ramIrTextField zurck. Es werden auerdem Initialisierungen
		vorgenommen.
		@return ramIrTextField
	*/
	private javax.swing.JTextField getramIrTextField() {
		if (ivjramIrTextField == null) {
			try {
				ivjramIrTextField = new javax.swing.JTextField();
				ivjramIrTextField.setName("ramIrTextField");
				ivjramIrTextField.setAutoscrolls(false);
				ivjramIrTextField.setText("0000");
				ivjramIrTextField.setColumns(4);
				ivjramIrTextField.setBounds(190, 44, 40, 24);
				ivjramIrTextField.setEditable(false);
				ivjramIrTextField.setHorizontalAlignment(javax.swing.JTextField.CENTER);
			} catch (java.lang.Throwable ivjExc) {
				handleException(ivjExc);
			}
		}
		return ivjramIrTextField;
	}

	
	private java.awt.Button ivjnextPhaseButton = null;
	
	
	private void connEtoC3(java.awt.event.ActionEvent arg1) {
		try {
			this.nextPhaseButton_ActionPerformed(arg1);
		} catch (java.lang.Throwable ivjExc) {
			handleException(ivjExc);
		}
	}
	
	/**
		Diese Methode generiert und liefert das Attribut
		nextPhaseButton zurck. Es werden auerdem Initialisierungen
		vorgenommen.
		@return nextPhaseButton
	*/
	private java.awt.Button getnextPhaseButton() {
		if (ivjnextPhaseButton == null) {
			try {
				ivjnextPhaseButton = new java.awt.Button();
				ivjnextPhaseButton.setName("nextPhaseButton");
				ivjnextPhaseButton.setBounds(24, 369, 87, 23);
				ivjnextPhaseButton.setLabel("NEXT PHASE");
			} catch (java.lang.Throwable ivjExc) {
				handleException(ivjExc);
			}
		}
		return ivjnextPhaseButton;
	}
	
	
	/**
 		Diese Methode wird aufgerufen, nachdem der Next-Phase Button
		gedrckt wurde. Sie ruft die passende Methode in
		<tt>ToyEmulatorKernel</tt> auf und bergibt ihr die Kontrolle
		ber den weiteren Ablauf.
		@param actionEvent: das ausgelste actionEvent
 	*/       
	public void nextPhaseButton_ActionPerformed(java.awt.event.ActionEvent actionEvent) {
		if(doNextStep) {
			callingKernel.step();
		} else {
			setArchTextFields();
			repaint();
			doNextStep=true;
		}
		repaint();
	}
}