import java.awt.*;
import java.awt.event.*;

/**
	Dies ist die Klasse zur Visualisierung der Programmsteuerelemente, der
	Registerinhalte und des Speichers. Smtliche grafischen Ausgabeelemente
	werden von hier verwaltet. Alle Grafikereignisse werden entweder direkt
	bearbeitet oder an die Klasse <tt>ToyEmulatorKernel</tt> weitergeleitet.
*/
public class ToyEmulatorGUI extends Frame implements 	ActionListener, 
																							WindowListener, 
																							TextListener,
																							MouseListener{

	//	private Klassenattribute
	private Button loadButton = null;			//  Load-Button
	private Button stepButton = null;			//  Step-Button
	private Button goButton = null;				//  Go-Button
	private Button endButton = null;			//  End-Button
	private Button archButton = null;			//  Show Architecture-Button
	private Button restartButton = null;			//  Restart-Button
	private Button setButton = null;			//  Set-Button
	private Panel contentsPane = null;			//  Container-Class fr die AWT-Komponente
	private Label accuLabel = null;				//  Akku-Beschriftung
	private Label pcLabel = null;				//  Programmcounter-Beschriftung
	private Label insnregLabel = null;			//  Befehlsregister-Beschriftung
	private Label mnemonicLabel = null;			//  Mnemonicfeld-Beschriftung
	private Label adressLabel = null;			//  Speicheradresse-Beschriftung
	private Label contentsLabel = null;			//  Speicherinhalt-Beschriftung
	private Label mnemonic2Label = null;			//  Mnemonicfeld ber Memoryfensterbeschriftung
	private Label adress2Label = null;			//  Speicheradresse-Beschriftung (2)
	private Label contentsHexLabel = null;			//  Speicherinhalt-Beschriftung (Hex)
	private Label contentsDecLabel = null;			//  Speicherinhalt-Beschriftung (Dec)
	private Label x01Label = null;				//  0x-Beschriftung vor Accutextfeld
	private Label x02Label = null;				//  0x-Beschriftung vor PC
	private Label x03Label = null;				//  0x-Beschriftung vor Insreg
	private Label messageLabel = null;			//  Hier werden bei Bedarf Programmausgaben verffentlicht
	private Label setLabel = null;				//  Set-Label vor Set-Feld
	private Label toLabel = null;				//  to-Label vor Adress-Feld
	private TextArea memoryTextArea = null;			//  Anzeigefeld fr Speicherinhalt (erste Hlfte)
	private TextArea memoryDataTextArea = null;		//  Anzeigefeld fr Speicherinhalt (zweite Hlfte)
	private TextField accuTextField = null;			//  Anzeigefeld fr Akkuinhalt
	private TextField accuDecTextField = null;		//  Anzeigefeld fr Akkuinhalt in decimal
	private TextField pcTextField = null;			//  Anzeigefeld fr Programcounter
	private TextField insnregTextField = null;		//  Anzeigefeld fr Befehlsregister
	private TextField mnemonicTextField = null;		//  Anzeigefeld fr Mnemonic
	private TextField stepTextField = null;			//  Anzeigefeld fr Stepweite
	private TextField setTextField = null;			//  Anzeigefeld fr Adresse
	private TextField toTextField = null; 			//  Anzeigefeld fr Wert
	private FileDialog loadFileDialog = null;		//  Filedialog zum Laden eines Quelltextes
	private ToyEmulatorKernel sourceKernel;			//  ToyEmulatorkernel, der GUI gestartet hat
	

	/**
		Der Konstruktor fr das GUI erzeugt ein neues Fenster und zeigt
		es an. Die Initialisierung wird von der Methode
		<tt>initialize</tt> bernommen, die von hier aus aufgerufen
		wird.
		@param sourceKernel: Dies ist der aufrufende ToyEmulatorKernel.
		Er wird hier gespeichert, um spter mit ihm kommunizieren zu
		knnen.
	*/
	public ToyEmulatorGUI(ToyEmulatorKernel sourceKernel) {

		super("Toy Emulator");				//	neues Fenster mit entsprechender berschrift
		initialize();					//	neues Fenster initialisieren (Layout, etc.)
		this.show();					//	neues Fenster anzeigen
		this.sourceKernel = sourceKernel;		//	aufrufenden Kernel vermerken
								//	--> 	das ist notwendig, um spter mit ihm 
								//	kommunizieren zu knnen
	}


	/**
		Initialisierung des Fensters mit korrekten Startwerten.
	*/
	private void initialize() {

		setName("ToyEmulatorGUI");					
		setLayout(new java.awt.GridLayout());	
		setLocation(100,75);						
		setSize(740, 440);	
		setResizable(false);
		setTitle("Toy Emulator");
		add(getContentsPane(), getContentsPane().getName());
		initConnections();
	}


	/**
		Hinzufgen der Listenerklassen zum Fenster
	*/
	private void initConnections() {

		this.addWindowListener(this);			//	Windowlistener
		getLoadButton().addActionListener(this);	//	Actionlistener
		getStepButton().addActionListener(this);	//		"
		getGoButton().addActionListener(this);		//		"
		getEndButton().addActionListener(this);		//		"
		getArchButton().addActionListener(this);	//  		"
		getRestartButton().addActionListener(this);	//		"
		getSetButton().addActionListener(this);		//		"
		getAccuTextField().addTextListener(this);	//	Textlistener
		getDecAccuTextField().addTextListener(this);	//	
		getPCTextField().addTextListener(this);		//		"
		getStepTextField().addTextListener(this);	//		"
		getSetTextField().addTextListener(this);	//		"
		getToTextField().addTextListener(this);		//		"
		getMemoryDataTextArea().addMouseListener(this);	//	MouseListener
	}


	/**
		Initialisierungsroutinen fr den LOAD-Knopf
		@return Zeiger auf den Knopf
	*/ 
	private Button getLoadButton() {

		if (loadButton == null) {
			try {
				loadButton = new Button();
				loadButton.setName("LoadButton");
				loadButton.setBounds(27, 343, 57, 23);
				loadButton.setLabel("LOAD");
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		};
		return loadButton;
	}


	/**
		Initialisierungsroutinen fr den STEP-Knopf
		@return Zeiger auf den Knopf
	*/
	private Button getStepButton() {

		if (stepButton == null) {
			try {
				stepButton = new Button();
				stepButton.setName("StepButton");
				stepButton.setBounds(114, 255, 57, 23);
				stepButton.setLabel("STEP");
				stepButton.setEnabled(false);
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		};
		return stepButton;
	}


	/**
		Initialisierungsroutinen fr den GO-Knopf
		@return Zeiger auf den Knopf
	*/
	public Button getGoButton() {

		if (goButton == null) {
			try {
				goButton = new Button();
				goButton.setName("GoButton");
				goButton.setBounds(27, 255, 57, 23);
				goButton.setLabel("GO");
				goButton.setEnabled(false);
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		};
		return goButton;
	}	


	/**
		Initialisierungsroutinen fr den END-Knopf
		@return Zeiger auf den Knopf
	*/
	private Button getEndButton() {

		if (endButton == null) {
			try {
				endButton = new Button();
				endButton.setName("EndButton");
				endButton.setBounds(201, 343, 57, 23);
				endButton.setLabel("END");
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		};
		return endButton;
	}


	/**
		Initialisierungsroutinen fr den ARCHITECTURE-Knopf
		@return Zeiger auf den Knopf
	*/
	private Button getArchButton() {

		if (archButton == null) {
			try {
				archButton = new Button();
				archButton.setName("ArchButton");
				archButton.setBounds(27, 299, 231, 23);
				archButton.setLabel("Show Architecture");
				archButton.setEnabled(false);
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		};
		return archButton;
	}


	/**
		Initialisierungsroutinen fr den Restart-Knopf
		@return Zeiger auf den Knopf
	*/
	private Button getRestartButton() {
		if (restartButton == null) {
			try {
				restartButton = new Button();
				restartButton.setName("RestartButton");
				restartButton.setBounds(114, 343, 57, 23);
				restartButton.setLabel("RELOAD");
				restartButton.setEnabled(false);
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		};
		return restartButton;
	}


	/**
		Initialisierungsroutinen fr den SET-Knopf
		@return Zeiger auf den Knopf
	*/
	private Button getSetButton() {

		if (setButton == null) {
			try {
				setButton = new Button();
				setButton.setName("SetButton");
				setButton.setBounds(520, 378, 57, 23);
				setButton.setLabel("SET");
				setButton.setEnabled(false);
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		};
		return setButton;
	}

	
	/**
		Initialisierungsroutinen fr das ACCU-Label
		@return Zeiger auf das Label
	*/
	private Label getAccuLabel() {

		if (accuLabel == null) {
			try {
				accuLabel = new Label();
				accuLabel.setName("AccuLabel");
				accuLabel.setText("Accumulator:");
				accuLabel.setBounds(32, 35, 85, 23);
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		}
		return accuLabel;
	}


	/**
		Initialisierungsroutinen fr das PC-Label
		@return Zeiger auf das Label
	*/
	private Label getPCLabel() {

		if (pcLabel == null) {
			try {
				pcLabel = new Label();
				pcLabel.setName("PCLabel");
				pcLabel.setText("PC:");
				pcLabel.setBounds(32, 80, 50, 23);
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		}
		return pcLabel;
	}


	/**
		Initialisierungsroutinen fr das INSREG-Label
		@return Zeiger auf das Label
	*/
	private Label getInsnregLabel() {

		if (insnregLabel == null) {
			try {
				insnregLabel = new Label();
				insnregLabel.setName("InsnregLabel");
				insnregLabel.setText("InsnReg:");
				insnregLabel.setBounds(32, 125, 60, 23);
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		}
		return insnregLabel;
	}


	/**
		Initialisierungsroutinen fr das MNEMONIC-Label
		@return Zeiger auf das Label
	*/
	private Label getMnemonicLabel() {

		if (mnemonicLabel == null) {
			try {
				mnemonicLabel = new Label();
				mnemonicLabel.setName("MnemonicLabel");
				mnemonicLabel.setText("Mnemonic:");
				mnemonicLabel.setBounds(32, 170, 70, 23);
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		}
		return mnemonicLabel;
	}


	/**
		Initialisierungsroutinen fr das ADRESS-Label
		@return Zeiger auf das Label
	*/
	private Label getAdressLabel() {

		if (adressLabel == null) {
			try {
				adressLabel = new Label();
				adressLabel.setName("AdressLabel:");
				adressLabel.setText("Address:");
				adressLabel.setBounds(304, 5, 63, 23);
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		}
		return adressLabel;
	}


	/**
		Initialisierungsroutinen fr das CONTENTS-Label
		@return Zeiger auf das Label
	*/
	private Label getContentsLabel() {

		if (contentsLabel == null) {
			try {
				contentsLabel = new Label();
				contentsLabel.setName("ContentsLabel");
				contentsLabel.setText("Contents(Hex):");
				contentsLabel.setBounds(396, 5, 100, 23);
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		}
		return contentsLabel;
	}


	/**
		Initialisierungsroutinen fr das 2. MNEMONIC-Label
		@return Zeiger auf das Label
	*/
	private Label getMnemonic2Label() {

		if (mnemonic2Label == null) {
			try {
				mnemonic2Label = new Label();
				mnemonic2Label.setName("Mnemonic2Label");
				mnemonic2Label.setText("Mnemonics:");
				mnemonic2Label.setBounds(520, 5, 78, 23);
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		}
		return mnemonic2Label;
	}


	/**
		Initialisierungsroutinen fr das 2. Adress-Label
		@return Zeiger auf das Label
	*/
	private Label getAdress2Label() {

		if (adress2Label == null) {
			try {
				adress2Label = new Label();
				adress2Label.setName("Adress2Label:");
				adress2Label.setText("Address:");
				adress2Label.setBounds(304, 225, 83, 23);
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		}
		return adress2Label;
	}


	/**
		Initialisierungsroutinen fr das CONTENTS(HEX)-Label
		@return Zeiger auf das Label
	*/
	private Label getContentsHexLabel() {

		if (contentsHexLabel == null) {
			try {
				contentsHexLabel = new Label();
				contentsHexLabel.setName("ContentsHexLabel");
				contentsHexLabel.setText("Contents(Hex):");
				contentsHexLabel.setBounds(386, 225, 100, 23);
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		}
		return contentsHexLabel;
	}


	/**
		Initialisierungsroutinen fr das CONTENTS(DEC)-Label
		@return Zeiger auf das Label
	*/
	private Label getContentsDecLabel() {

		if (contentsDecLabel == null) {
			try {
				contentsDecLabel = new Label();
				contentsDecLabel.setName("ContentsDecLabel");
				contentsDecLabel.setText("Contents(Dec):");
				contentsDecLabel.setBounds(520, 225, 100, 23);
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		}
		return contentsDecLabel;
	}	

	
	/**
		Initialisierungsroutinen fr das 1. 0x-Label
		@return Zeiger auf das Label
	*/
	private Label getx01Label() {

		if (x01Label == null) {
			try {
				x01Label = new Label();
				x01Label.setName("x01Label");
				x01Label.setText("0x");
				x01Label.setBounds(122, 35, 85, 23);
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		}
		return x01Label;
	}

	/**
		Initialisierungsroutinen fr das 2. 0x-Label
		@return Zeiger auf das Label
	*/
	private Label getx02Label() {

		if (x02Label == null) {
			try {
				x02Label = new Label();
				x02Label.setName("x02Label");
				x02Label.setText("0x");
				x02Label.setBounds(122, 80, 100, 23);
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		}
		return x02Label;
	}

	
	/**
		Initialisierungsroutinen fr das 3. 0x-Label
		@return Zeiger auf das Label
	*/
	private Label getx03Label() {

		if (x03Label == null) {
			try {
				x03Label = new Label();
				x03Label.setName("x03Label");
				x03Label.setText("0x");
				x03Label.setBounds(122, 125, 100, 23);
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		}
		return x03Label;
	}


	/**
		Initialisierungsroutinen fr das Message-Label. In diesem werden
		Statusinformationen des Programmes angezeigt.
		@return Zeiger auf das Label
	*/
	public Label getMessageLabel() {

		if (messageLabel == null) {
			try {
				messageLabel = new Label();
				messageLabel.setName("MessageLabel");
				messageLabel.setText("");
				messageLabel.setBounds(27, 380, 231, 23);
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		}
		return messageLabel;
	}


	/**
		Initialisierungsroutinen fr das Set-Label
		@return Zeiger auf das Label
	*/
	private Label getSetLabel() {

		if (setLabel == null) {
			try {
				setLabel = new Label();
				setLabel.setName("SetLabel");
				setLabel.setText("Set");
				setLabel.setBounds(295, 375, 50, 23);
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		}
		return setLabel;
	}


	/**
		Initialisierungsroutinen fr das To-Label
		@return Zeiger auf das Label
	*/
	private Label getToLabel() {

		if (toLabel == null) {
			try {
				toLabel = new Label();
				toLabel.setName("ToLabel");
				toLabel.setText("to:");
				toLabel.setBounds(390, 375, 50, 23);
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		}
		return toLabel;
	}

	
	/**
		Initialisierungsroutinen fr das Memory-Anzeigefeld
		@return Zeiger auf die TextArea
	*/
	private TextArea getMemoryTextArea() {

		if (memoryTextArea == null) {
			try {
				memoryTextArea = new TextArea("",1,1,TextArea.SCROLLBARS_VERTICAL_ONLY);
				memoryTextArea.setName("MemoryTextArea");
				memoryTextArea.setBackground(Color.white);
				memoryTextArea.setBounds(302, 34, 368, 190);
				memoryTextArea.setEditable(false);
				memoryTextArea.selectAll();
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		}
		return memoryTextArea;
	}


	/**
		Initialisierungsroutinen fr das 2. Memory-Anzeigefeld
		@return Zeiger auf die TextArea
	*/
	private TextArea getMemoryDataTextArea() {

		if (memoryDataTextArea == null) {
			try {
				memoryDataTextArea = new TextArea("",1,1,TextArea.SCROLLBARS_VERTICAL_ONLY);
				memoryDataTextArea.setName("MemoryDataTextArea");
				memoryDataTextArea.setBackground(Color.white);
				memoryDataTextArea.setBounds(302, 254, 368, 114);
				memoryDataTextArea.setEditable(false);
				memoryDataTextArea.selectAll();
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		}
		return memoryDataTextArea;
	}


	/**
		Initialisierungsroutinen fr das Accu-Textfeld
		@return Zeiger auf das TextField
	*/
	private TextField getAccuTextField() {

		if (accuTextField == null) {
			try {
				accuTextField = new TextField();
				accuTextField.setName("AccuTextField");
				accuTextField.setBackground(Color.white);
				accuTextField.setBounds(140, 35, 50, 30);
				accuTextField.setEditable(true);
				accuTextField.setColumns(4);
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		}
		return accuTextField;
	}


	/**
		Initialisierungsroutinen fr das decimale Accu-Textfeld
		@return Zeiger auf das TextField
	*/
	private TextField getDecAccuTextField() {

		if (accuDecTextField == null) {
			try {
				accuDecTextField = new TextField();
				accuDecTextField.setName("AccuDecTextField");
				accuDecTextField.setBackground(Color.white);
				accuDecTextField.setBounds(203, 35, 70, 30);
				accuDecTextField.setEditable(true);
				accuDecTextField.setColumns(6);		
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		}
		return accuDecTextField;
	}


	/**
		Initialisierungsroutinen fr das PC-Textfeld
		@return Zeiger auf das TextField
	*/
	private TextField getPCTextField() {

		if (pcTextField == null) {
			try {
				pcTextField = new TextField();
				pcTextField.setName("PCTextField");
				pcTextField.setBackground(Color.white);
				pcTextField.setBounds(140, 80, 50, 30);
				pcTextField.setEditable(true);
				pcTextField.setColumns(4);
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		}
		return pcTextField;
	}


	/**
		Initialisierungsroutinen fr das Instruktionsregister-Textfeld
		@return Zeiger auf das TextField
	*/
	private TextField getInsnregTextField() {

		if (insnregTextField == null) {
			try {
				insnregTextField = new TextField();
				insnregTextField.setName("InsnregTextField");
				insnregTextField.setBackground(Color.white);
				insnregTextField.setBounds(140, 125, 50, 30);
				insnregTextField.setEditable(false);
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		};
		return insnregTextField;
	}


	/**
		Initialisierungsroutinen fr das Mnemonic-Textfeld
		@return Zeiger auf das TextField
	*/
	private TextField getMnemonicTextField() {

		if (mnemonicTextField == null) {
			try {
				mnemonicTextField = new TextField();
				mnemonicTextField.setName("MnemonicTextField");
				mnemonicTextField.setBackground(Color.white);
				mnemonicTextField.setBounds(140, 170, 50, 30);
				mnemonicTextField.setEditable(false);
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		}
		return mnemonicTextField;
	}


	/**
		Initialisierungsroutinen fr das Step-Textfeld
		@return Zeiger auf das TextField
	*/
	private TextField getStepTextField() {

		if (stepTextField == null) {
			try {
				stepTextField = new TextField();
				stepTextField.setName("StepTextField");
				stepTextField.setBackground(Color.white);
				stepTextField.setBounds(201, 252, 60, 30);
				stepTextField.setEditable(true);
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		}
		return stepTextField;
	}


	/**
		Initialisierungsroutinen fr das Set-Textfeld
		@return Zeiger auf das TextField
	*/
	private TextField getSetTextField() {

		if (setTextField == null) {
			try {
				setTextField = new TextField();
				setTextField.setName("SetTextField");
				setTextField.setBackground(Color.white);
				setTextField.setBounds(320, 375, 45, 30);
				setTextField.setEditable(true);
				setTextField.setColumns(4);
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		}
		return setTextField;
	}


	/**
		Initialisierungsroutinen fr das To-Textfeld
		@return Zeiger auf das TextField
	*/
	private TextField getToTextField() {

		if (toTextField == null) {
			try {
				toTextField = new TextField();
				toTextField.setName("ToTextField");
				toTextField.setBackground(Color.white);
				toTextField.setBounds(415, 375, 45, 30);
				toTextField.setEditable(true);
				toTextField.setColumns(4);
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		}
		return toTextField;
	}


	/**
		Hier werden alle Einzelkomonenten in einem Container
		zusammengefgt. Dieser Container ist ein Panel, das anschlieend
		zurckgeliefert wird.
		@return Zeiger auf das Hauptpanel
	*/

	private Panel getContentsPane() {

		if (contentsPane == null) {
			try {
				contentsPane = new Panel();
				contentsPane.setName("ContentsPane");
				contentsPane.setLayout(null);
				getContentsPane().add(getGoButton(), getGoButton().getName());
				getContentsPane().add(getStepButton(), getStepButton().getName());
				getContentsPane().add(getStepTextField(), getStepTextField().getName());
				getContentsPane().add(getArchButton(), getArchButton().getName());
				getContentsPane().add(getLoadButton(), getLoadButton().getName());
				getContentsPane().add(getRestartButton(), getRestartButton().getName());
				getContentsPane().add(getEndButton(), getEndButton().getName());
				getContentsPane().add(getSetTextField(), getSetTextField().getName());
				getContentsPane().add(getToTextField(), getToTextField().getName());
				getContentsPane().add(getSetButton(), getSetButton().getName());
				getContentsPane().add(getMemoryTextArea(), getMemoryTextArea().getName());
				getContentsPane().add(getMemoryDataTextArea(), getMemoryDataTextArea().getName());
				getContentsPane().add(getAccuTextField(), getAccuTextField().getName());
				getContentsPane().add(getDecAccuTextField(), getDecAccuTextField().getName());
				getContentsPane().add(getPCTextField(), getPCTextField().getName());
				getContentsPane().add(getAccuLabel(), getAccuLabel().getName());
				getContentsPane().add(getPCLabel(), getPCLabel().getName());
				getContentsPane().add(getInsnregTextField(), getInsnregTextField().getName());
				getContentsPane().add(getInsnregLabel(), getInsnregLabel().getName());
				getContentsPane().add(getMnemonicTextField(),getMnemonicTextField().getName());
				getContentsPane().add(getMnemonicLabel(), getMnemonicLabel().getName());	
				getContentsPane().add(getAdressLabel(), getAdressLabel().getName());
				getContentsPane().add(getContentsLabel(), getContentsLabel().getName());
				getContentsPane().add(getMnemonic2Label(), getMnemonic2Label().getName());
				getContentsPane().add(getAdress2Label(), getAdress2Label().getName());
				getContentsPane().add(getContentsHexLabel(), getContentsHexLabel().getName());
				getContentsPane().add(getContentsDecLabel(), getContentsDecLabel().getName());
				getContentsPane().add(getx01Label(), getx01Label().getName());
				getContentsPane().add(getx02Label(), getx02Label().getName());
				getContentsPane().add(getx03Label(), getx03Label().getName());
				getContentsPane().add(getMessageLabel(), getMessageLabel().getName());
				getContentsPane().add(getSetLabel(), getSetLabel().getName());
				getContentsPane().add(getToLabel(), getToLabel().getName());
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		}
		return contentsPane;
	}

	/**
		Diese Methode reagiert auf Aktivierung von Programmknpfen durch
		den Benutzer durch Aufruf der entsprechenden Behandlungsroutine.
		@param e: Das ausgelste ActionEvent
	*/
	public void actionPerformed(ActionEvent e) {

		if ((e.getSource() == getLoadButton()) ) {		//	Load-Button gedrckt ?!
			connectEventToLoadFileDialog();			// 	--> FileDialog starten
		}
		if ((e.getSource() == getStepButton()) ) {		//	Step-Button gedrckt
			connectEventToStep();				//	--> Quellcode weiterbearbeiten
		}
		if ((e.getSource() == getGoButton()) ) {		//	Go-Button gedrckt
			connectEventToGo();				//	--> ganzen Quellcode abarbeiten
		}
		if ((e.getSource() == getEndButton()) ) {		//	End-Button gedrckt
			connectEventToEndToyEmulator();			//	--> Programm beenden
		}
		if ((e.getSource() == getArchButton()) ) {		//	Arch-Button gedrckt
			connectEventToArch();				//	--> Architektur zeigen oder verstecken
		}
		if ((e.getSource() == getRestartButton()) ) {		//	Restart-Button gedrckt
			connectEventToReset();				// --> Programm von vorne beginnen
		}
		if ((e.getSource() == getSetButton()) ) {		//	Set-Button gedrckt
			connectEventToSet();				// --> Datum setzen
		}
	}


	/**
		Diese Methode startet den FileDialog zum Einlesen eines
		Toy-Quellcodes. Alles weitere wird von einer entsprechenden
		Aktionsmethode im <tt>ToyEmulatorKernel</tt> geregelt.
	*/
	private void connectEventToLoadFileDialog() {

		try {
			getLoadFileDialog().show();

			// Der Kernel erhlt die geladene Datei, um sie in den Speicher zu schreiben
			sourceKernel.loadFile(loadFileDialog.getDirectory()+loadFileDialog.getFile());

			// Aktivieren einiger Knpfe
			goButton.setEnabled(true);
			stepButton.setEnabled(true);
			restartButton.setEnabled(true);
			setButton.setEnabled(true);
			archButton.setEnabled(true);
		} catch (java.lang.Throwable exc) {
			handleException(exc);
		}
	}


	/**
		Diese Methode wird nach dem Drcken des STEP-Knopfes aktiviert.
		Sie ruft eine Methode im <tt>ToyEmulatorKernel</tt>
		auf, die dann den nchsten Programmschritt bearbeitet.
	*/
	private void connectEventToStep() {

		sourceKernel.step();		//	Die notwendigen Berechnungen fhrt die Logik des Kernels aus
	}


	/**
		Diese Methode wird nach dem Drcken des GO-Knopfes aktiviert.
		Sie ruft eine Methode im <tt>ToyEmulatorKernel</tt>
		auf, die dann alle Programmschritte bis zum Prgrammende oder
		Abbruch durch den Benutzer abarbeitet.
	*/
	private void connectEventToGo() {
		goButton.setLabel("STOP");
		sourceKernel.setgomode(true);
		messageLabel.setText("Click any mouse button to STOP !");
		sourceKernel.go();		//	Auch hier werden alle Rechenschritte dem Kernel berlassen
	}


	/**
		Diese Methode wird nach dem Drcken des END-Knopfes aktiviert.
		Anschlieend wird das Programm verlassen.
	*/
	private void connectEventToEndToyEmulator() {

		try {
			this.dispose();
			System.exit(0);
		} catch (java.lang.Throwable exc) {
			handleException(exc);
		}
	}


	/**
		Diese Methode wird nach dem Drcken des Show Architecture-Knopfes aktiviert.
		Sie ruft eine Methode im <tt>ToyEmulatorKernel</tt>
		auf, die dann entweder das Architektur-Fenster zeigt oder
		versteckt.
	*/
	private void connectEventToArch() {

		sourceKernel.arch();
	}

	
	/**
		Diese Methode wird nach dem Drcken des RESET-Knopfes aktiviert.
		Sie ruft eine Methode im <tt>ToyEmulatorKernel</tt>
		auf, die das aktuelle Toy-Programm und den Speicher auf seine Anfangswerte
		zurcksetzt.
	*/
	private void connectEventToReset() {

		sourceKernel.reset();
	}


	/**
		Diese Methode wird nach dem Drcken des SET-Knopfes aktiviert.
		Sie ruft eine Methode im <tt>ToyEmulatorKernel</tt>
		auf, die die genderten Werte fr ein Speicherfeld in den
		Toy-Speicher bertrgt.
	*/
	private void connectEventToSet() {

		String Temp;
		int pos;

		Temp=toTextField.getText();
		for(int i=Temp.length();i<4;i++)
			Temp = "0" + Temp;
			
		pos=memoryDataTextArea.getCaretPosition();
		sourceKernel.setMemoryValue(Integer.parseInt(setTextField.getText(),16),Temp);
		memoryDataTextArea.select(pos-((pos-1)%24),pos-((pos-1)%24)+24);
	}

	
	/**
		Diese Methode liefert den FileDialog zum Einlesen eines
		Toy-Quellcodes zurck.
		@return Filedialog
	*/
	private FileDialog getLoadFileDialog() {

		if (loadFileDialog == null) {
			try {
				loadFileDialog = new FileDialog(this);
				loadFileDialog.setName("LoadFileDialog");
				loadFileDialog.setLayout(null);
			} catch (java.lang.Throwable exc) {
				handleException(exc);
			}
		}
		return loadFileDialog;
	}


	/**
		Diese Methode wird aufgerufen wenn sich ein Wert in einem
		berwachten Textfeld ndert und dient damit als Listenermethode 
		fr nderungen in den Textfeldern.
		@param e: das ausgelste TextEvent
	*/
	public void textValueChanged(TextEvent e) {

		if (e.getSource() == accuTextField) {
			connectEventToNewAccuValue();
		}
		if (e.getSource() == accuDecTextField) {
			connectEventToNewAccuDecValue();
		}
		if (e.getSource() == pcTextField) {
			connectEventToNewPCValue();
		}
		if (e.getSource() == stepTextField) {
			connectEventToNewStepValue();
		}
		if (e.getSource() == setTextField) {
			connectEventToNewSetValue();
		}
		if (e.getSource() == toTextField) {
			connectEventToNewToValue();
		}
	}


	/**
		Diese Methode wird ausgefhrt, wenn vom Benutzer ein neuer
		ACCU-Wert in das entsprechende Textfeld eingetragen wurde. Es
		wird anschlieend eine Methode im <tt>ToyEmulatorKernel</tt>
		aufgerufen, die sich um die weitere Bearbeitung dieses
		Ereignisses kmmert. Falls das Textfeld leer ist, wird der Wert
		0 angenommen. Auch das accuDecTextField wird mit dem aktuellen
		Wert aktualisiert.
	*/
	private void connectEventToNewAccuValue() {

		String newAccuValue = accuTextField.getText();
		String newAccuDecValue;
		int curAccuDecValue;
		
		if (newAccuValue.length() > 4) {
			newAccuValue=newAccuValue.substring(0,4);
			accuTextField.setText(newAccuValue);
		}

		//	Akkuwert an ToyEmulatorKernel bergeben
		if (newAccuValue.equals("")) {
			newAccuValue="0000";
			sourceKernel.setAccuValue(newAccuValue);
		}
		else
			sourceKernel.setAccuValue(accuTextField.getText());

		// DecAccu-Fenster bei Bedarf aktualisieren
		newAccuDecValue = ""+Integer.parseInt(newAccuValue,16);
		
		if(accuDecTextField.getText().equals(""))
			curAccuDecValue=0;
		else
			curAccuDecValue=Integer.parseInt(accuDecTextField.getText(),10);
		if(Integer.parseInt(newAccuDecValue,10)!=curAccuDecValue)
			accuDecTextField.setText(newAccuDecValue);
		
		// Architekturfenster aktualisieren
		if(sourceKernel.getArchGUI()!=null)
		{
			sourceKernel.getArchGUI().getaccuAluTextField().setText(accuTextField.getText());	
			sourceKernel.getArchGUI().repaint();
		}
	}

	/**
		Diese Methode wird ausgefhrt, wenn vom Benutzer ein neuer
		dezimaler ACCU-Wert in das entsprechende Textfeld eingetragen wurde. Es
		wird anschlieend eine Methode im <tt>ToyEmulatorKernel</tt>
		aufgerufen, die sich um die weitere Bearbeitung dieses
		Ereignisses kmmert. Falls das Textfeld leer ist, wird der Wert
		0 angenommen. Auch das accuTextField wird mit dem aktuellen
		Wert aktualisiert.
	*/
	private void connectEventToNewAccuDecValue() {

		String newAccuDecValue = accuDecTextField.getText();
		String newAccuValue;
		TArchGUI archGUI;
		int curAccuValue;
		
		if(newAccuDecValue.equals("")) {
			newAccuDecValue="00000";
			sourceKernel.setAccuValue(WholeHexNumber.toHex(Integer.parseInt("00000",10)).toString());
		} else {
			if (newAccuDecValue.length() > 5) {
				newAccuDecValue=newAccuDecValue.substring(0,5);
				accuDecTextField.setText(newAccuDecValue);
			}
		
			if (Integer.parseInt(newAccuDecValue,10)>65535) {
				newAccuDecValue="65535";
				accuDecTextField.setText(newAccuDecValue);
			}

			// Akkuwert an ToyEmulatorKernel bergeben
			sourceKernel.setAccuValue(WholeHexNumber.toHex(Integer.parseInt(newAccuDecValue,10)).toString());
		}
		
		// Accu-Fenster bei Bedarf aktualisieren
		newAccuValue = WholeHexNumber.toHex(Integer.parseInt(newAccuDecValue,10)).toString();
		
		if(accuTextField.getText().equals(""))
			curAccuValue=0;
		else
			curAccuValue=Integer.parseInt(accuTextField.getText(),16);
		if(Integer.parseInt(newAccuValue,16)!=curAccuValue)
			accuTextField.setText(newAccuValue);		
		
		// Architekturfenster aktualisieren
		archGUI=sourceKernel.getArchGUI();
		if(archGUI!=null)
		{
			archGUI.getaccuAluTextField().setText(WholeHexNumber.toHex(Integer.parseInt(newAccuDecValue,10)).toString());
			sourceKernel.getArchGUI().repaint();
		}
	}


	/**
		Diese Methode wird ausgefhrt, wenn vom Benutzer ein neuer
		Befehlszhler-Wert in das entsprechende Textfeld eingetragen wurde. Es
		wird anschlieend eine Methode im <tt>ToyEmulatorKernel</tt>
		aufgerufen, die sich um die weitere Bearbeitung dieses
		Ereignisses kmmert.
	*/
	private void connectEventToNewPCValue() {

		TArchGUI archGUI=sourceKernel.getArchGUI();
		String newPCValue = pcTextField.getText();
		
		if(newPCValue.equals("")) return;
		if (newPCValue.length() > 4) pcTextField.setText(newPCValue.substring(0,4));
		
		//	PCwert an ToyEmulatorKernel bergeben
		sourceKernel.setPCValue(pcTextField.getText());

		// Architekturfenster aktualisieren
		if(archGUI!=null) {
			archGUI.setDoNextStep(true);
			archGUI.setArchTextFields();
			archGUI.setDoNextStep(false);
		}
	}


	/**
		Diese Methode wird ausgefhrt, wenn vom Benutzer ein neuer
		STEP-Wert in das entsprechende Textfeld eingetragen wurde. Es
		wird anschlieend eine Methode im <tt>ToyEmulatorKernel</tt>
		aufgerufen, die sich um die weitere Bearbeitung dieses
		Ereignisses kmmert.
	*/
	private void connectEventToNewStepValue() {

		int newStep;

		try {
			if(stepTextField.getText().equals("")) return;
			newStep = Integer.parseInt(stepTextField.getText(),10);
			if (newStep < 1) {
				stepTextField.setText("1");
				newStep = 1;
			}
		
			//	Stepwert an ToyEmulatorKernel bergeben
			sourceKernel.setStepWidth(newStep);
		}
		catch (java.lang.NumberFormatException exc) {
			stepTextField.setText(Integer.toString(sourceKernel.getStepWidth()));
		}
	}


	/**
		Diese Methode wird ausgefhrt, wenn vom Benutzer ein neuer
		Speicherfeld-Wert in das entsprechende Textfeld eingetragen wurde. Es
		wird nur geprft, ob dieser Wert im korrekten Format vorliegt.
	*/
	private void connectEventToNewSetValue() {

		String newSetValue = setTextField.getText();
		if (newSetValue.length() > 4) setTextField.setText(newSetValue.substring(0,4));
	}


	/**
		Diese Methode wird ausgefhrt, wenn vom Benutzer ein neuer
		Speicherfeld-Wert in das entsprechende Textfeld eingetragen wurde. Es
		wird nur geprft, ob dieser Wert im korrekten Format vorliegt.
	*/
	private void connectEventToNewToValue() {

		String newToValue = toTextField.getText();
		if (newToValue.length() > 4) toTextField.setText(newToValue.substring(0,4));
	}
	
	
	/**
		Diese Methode wird aufgerufen, wenn der Benutzer mit der Maus
		innerhalb des Programmes etwas anklickt. Es wird anschlieend
		die entsprechende Bearbeitungsmethode ausgefhrt.
		@param e: das ausgelste Mausereignis
	*/
	public void mouseClicked(MouseEvent e)
	{
		if (e.getSource() == memoryDataTextArea) {
			connectEventToMemoryDataTextAreaClicked();
		}
	}


 	/**
		Diese Methode ist nur vorhanden, da sie von der implementierten
		Schnittstelle gefordert wird, hat aber darber hinaus keine
		Funktion.
	*/
	public void mouseEntered(MouseEvent e) 
	{
		;
	}

 	/**
		Diese Methode ist nur vorhanden, da sie von der implementierten
		Schnittstelle gefordert wird, hat aber darber hinaus keine
		Funktion.
	*/
	public void mouseExited(MouseEvent e) 
	{
		;
	}

	/**
		Diese Methode ist nur vorhanden, da sie von der implementierten
		Schnittstelle gefordert wird, hat aber darber hinaus keine
		Funktion.
	*/
	public void mousePressed(MouseEvent e) 
	{
		;
	}

	/**
		Diese Methode ist nur vorhanden, da sie von der implementierten
		Schnittstelle gefordert wird, hat aber darber hinaus keine
		Funktion.
	*/
	public void mouseReleased(MouseEvent e) 
	{
		;
	}

	
	/**
		Diese Methode wird aufgerufen, wenn der Benutzer mit der Maus
		auf das Textfeld mit der 2. Speicherhlfte klickt. Es werden die
		Daten in der geklickten Zeile in die darunterliegenden
		Textfelder bertragen und die angeclickte Zeile wird markiert.
	*/
	private void connectEventToMemoryDataTextAreaClicked()
	{
		int pos,cellNr;
		String adr="";

		pos=memoryDataTextArea.getCaretPosition();
		setTextField.setText(Integer.toString(pos));
		cellNr=(pos-1)/24+sourceKernel.getToyMemory().getMaxAdr()/2;
		toTextField.setText(sourceKernel.getToyMemory().getCell(cellNr));
		adr = WholeHexNumber.toHex(cellNr).toString();
		setTextField.setText(adr);
		memoryDataTextArea.select(pos-((pos-1)%24),pos-((pos-1)%24)+24);
	}

	
	/**
		Diese Methode wird aufgerufen, wenn das Hauptfenster des
		Programmes geschlossen werden soll. Bevor dies geschieht wird
		eine andere Methode aufgerufen, die das Programm beendet
		@param e: das bermittelte Window-Event
	*/ 
	public void windowClosing(WindowEvent e) {

		if (e.getSource() == this) {
			connectEventToEndToyEmulator();
		}
	}	


	/**
		Diese Methode ist nur vorhanden, da sie von der implementierten
		Schnittstelle gefordert wird, hat aber darber hinaus keine
		Funktion.
	*/
	public void windowActivated(java.awt.event.WindowEvent e) {

		;
	}

	/**
		Diese Methode ist nur vorhanden, da sie von der implementierten
		Schnittstelle gefordert wird, hat aber darber hinaus keine
		Funktion.
	*/
	public void windowClosed(java.awt.event.WindowEvent e) {

		;
	}

	/**
		Diese Methode ist nur vorhanden, da sie von der implementierten
		Schnittstelle gefordert wird, hat aber darber hinaus keine
		Funktion.
	*/
	public void windowDeactivated(java.awt.event.WindowEvent e) {

		;
	}

	/**
		Diese Methode ist nur vorhanden, da sie von der implementierten
		Schnittstelle gefordert wird, hat aber darber hinaus keine
		Funktion.
	*/
	public void windowDeiconified(java.awt.event.WindowEvent e) {

		;
	}

	/**
		Diese Methode ist nur vorhanden, da sie von der implementierten
		Schnittstelle gefordert wird, hat aber darber hinaus keine
		Funktion.
	*/
	public void windowIconified(java.awt.event.WindowEvent e) {

		;
	}

	/**
		Diese Methode ist nur vorhanden, da sie von der implementierten
		Schnittstelle gefordert wird, hat aber darber hinaus keine
		Funktion.
	*/
	public void windowOpened(java.awt.event.WindowEvent e) {

		;
	}


	/** 
		Dies ist ein allgemeiner ExceptionHandler, der eine Fehlermeldung
		ausgibt, falls eine nicht abgefangene Ausnahmebedingung ausgelst wurde
	*/
	private void handleException(Throwable exception) {

		System.out.println("--------- NICHT ABGEFANGENE AUSNAHMEBEDINGUNG ---------");
		exception.printStackTrace(System.out);
	}


	/**
		Diese Methode ermglicht den Zugriff auf das private Attribut
		<tt>AccuTextField<tt>.
		@param text: neuer Text fr <tt>AccuTextField<tt>.
	*/
	public void setAccuTextField(String text) {

		accuTextField.setText(text);
	}

	
	/**
		Diese Methode ermglicht den Zugriff auf das private Attribut
		<tt>AccuDecTextField<tt>.
		@param text: neuer Text fr <tt>AccuDecTextField<tt>.
	*/
	public void setAccuDecTextField(String text) {

		accuDecTextField.setText(text);
	}


	/**
		Diese Methode ermglicht den Zugriff auf das private Attribut
		<tt>PCTextField<tt>.
		@param text: neuer Text fr <tt>PCTextField<tt>.
	*/
	public void setPCTextField(String text) {

		pcTextField.setText(text);
		setHighlight(text);
	}


	/**
		Diese Methode ermglicht den Zugriff auf das private Attribut
		<tt>InsregTextField<tt>.
		@param text: neuer Text fr <tt>InsregTextField<tt>.
	*/
	public void setInsnregTextField(String text) {

		insnregTextField.setText(text);
	}


	/**
		Diese Methode ermglicht den Zugriff auf das private Attribut
		<tt>MnemonicTextField<tt>.
		@param text: neuer Text fr <tt>MnemonicTextField<tt>.
	*/
	public void setMnemonicTextField(String text) {

		mnemonicTextField.setText(text);
	}

	
	/**
		Diese Methode ermglicht den Zugriff auf das private Attribut
		<tt>StepTextField<tt>.
		@param text: neuer Text fr <tt>StepTextField<tt>.
	*/
	public void setStepTextField(String text) {

		stepTextField.setText(text);
	}
	
	/**
		Diese Methode ermglicht den Zugriff auf das private Attribut
		<tt>MemoryTextArea<tt>.
		@param text: neuer Text fr <tt>MemoryTextArea<tt>.
	*/
	public void setMemoryTextArea(String text) {

		memoryTextArea.setText(text);
	}

	/**
		Diese Methode ermglicht den Zugriff auf das private Attribut
		<tt>MemoryDataTextArea<tt>.
		@param text: neuer Text fr <tt>MemoryDataTextArea<tt>.
	*/
	public void setMemoryDataTextArea(String text) {

		memoryDataTextArea.setText(text);
	}
	

	/**
		Diese Methode ermglicht den Zugriff auf das private Attribut
		<tt>archButton<tt>.
		@param text: neuer Text fr <tt>archButton<tt>.
	*/
	public void setArchButtonLabel(String text) {

		archButton.setLabel(text);
	}
	

	/**
		Hier wird die aktuelle Programmzeile markiert und damit
		hervorgehoben.
		@param starttext: der Beginn der aktuellen Zeile
	*/
	public void setHighlight(String starttext)
	{
		int startpos, endpos;
		String alltext;

		memoryTextArea.select(0,0);
		
		alltext=memoryTextArea.getText();
		startpos = alltext.indexOf("\n     " + starttext);
		endpos=alltext.indexOf("\n",startpos+1);

		memoryTextArea.select(startpos+2,endpos);
		
	}

}