/**
	Die Klasse ToyMemory hat die Aufgabe den Speicher des Toy-Rechners zu 
	simulieren. Die Daten werden in einem Feld von Strings gehalten.
	Methoden zum Abfragen von Speicherzellen werden bereitgestellt.
*/
public class ToyMemory {

	/**
		Der verwendete Konstruktor legt einen Speicherbereich an, dessen
		Gre durch die Konstante <tt>MAXADR</tt> festgelegt wird. Jede
		Zelle wird mit dem Wert 0 initialisiert
	*/ 
	public ToyMemory() {

		for (int i = 0; i < MAXADR; i++) memory[i] = 0;
	}


	/**
		Diese Methode dient zum Zurcksetzen des Speichers auf seinen
		Anfangszustand (d.h. jede Zelle erhlt erneut den Wert 0).
	*/
	public void resetMemory() {

		for (int i = 0; i < MAXADR; i++) memory[i] = 0;
	}


	/**
		Hier wird eine Speicherzelle mit einem Wert belegt.
		@param adr: Nummer der zu fllenden Zelle
		@param content: Zuknftiger Wert der Zelle
	*/
	public void setCell(int adr, String content) {

		memory[adr] = Integer.parseInt(content,16);
	}


	/**
		Diese Methode gibt den Zelleninhalt einer Zelle zurck.
		@param adr: Nummer der zurckzugebenden Zelle
		@return Inhalt der zurckzugebenden Zelle
	*/
	public String getCell(int adr) {

		String temp;		
		
		temp=Integer.toString(memory[adr],16);
		for(int i=temp.length();i<4;i++)
			temp="0"+temp;
		return temp;
	}


	/**
		Diese Methode copiert den gesamten Speicher in einen String und
		bringt ihn in das richtige Ausgabeformat. Dabei wird u.a. die
		Mnemonic oder die Dezimaldarstellung der Zahlen hinzugefgt.
		@param firsthalf: falls dieser Wert true ist, wird die erste
		Speicherhlfte zurckgegeben, ansonsten die zweite, wobei beide
		unterschiedliche Formatierungen erhalten
		@return Speicherinhalt als String
	*/
	public String toString(boolean firsthalf) {
		StringBuffer mem;
		String temp;
		int tempadr,j=0;

		if(firsthalf) {
			mem = new StringBuffer(27000);
			for (int i = 0; i < MAXADR/2; i++) {
				String adr = (WholeHexNumber.toHex(i)).toString();
				temp = Integer.toString(memory[i],16);
				mem.append("\n     " + adr + "\t\t");
				if(memory[i]!=0) for(j=0;j<4-temp.length();j++) temp="0" + temp;
				mem.append(temp);
				tempadr = Integer.parseInt(temp.substring(0,1),16); 
				if(memory[i]!=0) {
					mem.append("\t\t" + ToyEmulatorKernel.getMnemonicFromInt(tempadr));
					if(tempadr<8) mem.append("(" + temp.substring(1) + ")");
				}
			} 		
		} else {
			mem = new StringBuffer(49200);
			for (int i = MAXADR/2; i < MAXADR; i++) {
				String adr = (WholeHexNumber.toHex(i)).toString();
				temp=Integer.toString(memory[i],16);
				
				mem.append("\n     " + adr + "\t\t");
				// Falls in einer Zeile keine Null steht, noch mit Nullen auffllen
				if(memory[i]!=0) for(j=temp.length();j<4;j++) mem.append("0");
				
				mem.append(temp);
				
				// Falls in einer Zelle 0 steht, drei Leerzeichen anfgen
				if(memory[i]==0) mem.append("   ");				
				
				tempadr = memory[i]; 

				// fr negative Zahlen korrekte Darstellung whlen
				if(tempadr>32767) {
					tempadr = 65536 - tempadr;
					tempadr *= -1;
				}
				
				// Tabs
				mem.append("\t\t");
				
				// Fhrende Lehrzeichen hinzufgen
				for(j=Integer.toString(tempadr).length();j<6;j++)
					 mem.append(" ");
				
				// Decimalwert hinzufgen
				mem.append(Integer.toString(tempadr));
			}
		}
		return mem.toString();
	}

	/**
		Diese Methode gibt die aktuelle Speichergre zurck
		@return Speichergre als Integer
	*/
	public int getMaxAdr() {
		return MAXADR;
	}

	private final int MAXADR = 4096;		//	Konstante beschrnkt Speichergre
	private int[] memory = new int[MAXADR];	//	Speicherfeld
}